@@ -1,5 +1,5 @@
 ﻿/**
- * @license Copyright (c) 2003-2014, CKSource - Frederico Knabben. All rights reserved.
+ * @license Copyright (c) 2003-2015, CKSource - Frederico Knabben. All rights reserved.
  * For licensing, see LICENSE.md or http://ckeditor.com/license
  */
 
@@ -84,8 +84,28 @@
 	// #### checkSelectionChange : END
 
 	var isVisible = CKEDITOR.dom.walker.invisible( 1 );
+
+	// May absorb the caret if:
+	// * is a visible node,
+	// * is a non-empty element (this rule will accept elements like <strong></strong> because they
+	//	they were not accepted by the isVisible() check, not not <br> which cannot absorb the caret).
+	//	See #12621.
+	function mayAbsorbCaret( node ) {
+		if ( isVisible( node ) )
+			return true;
+
+		if ( node.type == CKEDITOR.NODE_ELEMENT && !node.is( CKEDITOR.dtd.$empty ) )
+			return true;
+
+		return false;
+	}
+
 	function rangeRequiresFix( range ) {
-		function isTextCt( node, isAtEnd ) {
+		// Whether we must prevent from absorbing caret by this context node.
+		// Also checks whether there's an editable position next to that node.
+		function ctxRequiresFix( node, isAtEnd ) {
+			// It's ok for us if a text node absorbs the caret, because
+			// the caret container element isn't changed then.
 			if ( !node || node.type == CKEDITOR.NODE_TEXT )
 				return false;
 
@@ -100,17 +120,18 @@
 
 		var ct = range.startContainer;
 
-		var previous = range.getPreviousNode( isVisible, null, ct ),
-			next = range.getNextNode( isVisible, null, ct );
+		var previous = range.getPreviousNode( mayAbsorbCaret, null, ct ),
+			next = range.getNextNode( mayAbsorbCaret, null, ct );
 
-		// Any adjacent text container may absorb the cursor, e.g.
+		// Any adjacent text container may absorb the caret, e.g.
 		// <p><strong>text</strong>^foo</p>
 		// <p>foo^<strong>text</strong></p>
 		// <div>^<p>foo</p></div>
-		if ( isTextCt( previous ) || isTextCt( next, 1 ) )
+		if ( ctxRequiresFix( previous ) || ctxRequiresFix( next, 1 ) )
 			return true;
 
 		// Empty block/inline element is also affected. <span>^</span>, <p>^</p> (#7222)
+		// If you found this line confusing check #12655.
 		if ( !( previous || next ) && !( ct.type == CKEDITOR.NODE_ELEMENT && ct.isBlockBoundary() && ct.getBogus() ) )
 			return true;
 
@@ -149,25 +170,20 @@
 
 			// Text selection position might get mangled by
 			// subsequent dom modification, save it now for restoring. (#8617)
-			if ( keepSelection !== false )
-			{
+			if ( keepSelection !== false ) {
 				var bm,
-					doc = element.getDocument(),
-					sel = doc.getSelection().getNative(),
+					sel = element.getDocument().getSelection().getNative(),
 					// Be error proof.
 					range = sel && sel.type != 'None' && sel.getRangeAt( 0 );
 
 				if ( fillingChar.getLength() > 1 && range && range.intersectsNode( fillingChar.$ ) ) {
-					bm = [ sel.anchorOffset, sel.focusOffset ];
+					bm = createNativeSelectionBookmark( sel );
 
 					// Anticipate the offset change brought by the removed char.
 					var startAffected = sel.anchorNode == fillingChar.$ && sel.anchorOffset > 0,
 						endAffected = sel.focusNode == fillingChar.$ && sel.focusOffset > 0;
-					startAffected && bm[ 0 ]--;
-					endAffected && bm[ 1 ]--;
-
-					// Revert the bookmark order on reverse selection.
-					isReversedSelection( sel ) && bm.unshift( bm.pop() );
+					startAffected && bm[ 0 ].offset--;
+					endAffected && bm[ 1 ].offset--;
 				}
 			}
 
@@ -176,13 +192,9 @@
 			// invisible char from it.
 			fillingChar.setText( replaceFillingChar( fillingChar.getText() ) );
 
-			// Restore the bookmark.
+			// Restore the bookmark preserving selection's direction.
 			if ( bm ) {
-				var rng = sel.getRangeAt( 0 );
-				rng.setStart( rng.startContainer, bm[ 0 ] );
-				rng.setEnd( rng.startContainer, bm[ 1 ] );
-				sel.removeAllRanges();
-				sel.addRange( rng );
+				moveNativeSelectionToBookmark( element.getDocument().$, bm );
 			}
 		}
 	}
@@ -194,52 +206,22 @@
 		} );
 	}
 
-	function isReversedSelection( sel ) {
-		if ( !sel.isCollapsed ) {
-			var range = sel.getRangeAt( 0 );
-			// Potentially alter an reversed selection range.
-			range.setStart( sel.anchorNode, sel.anchorOffset );
-			range.setEnd( sel.focusNode, sel.focusOffset );
-			return range.collapsed;
-		}
+	function createNativeSelectionBookmark( sel ) {
+		return [
+			{ node: sel.anchorNode, offset: sel.anchorOffset },
+			{ node: sel.focusNode, offset: sel.focusOffset }
+		];
 	}
 
-	// Read the comments in selection constructor.
-	function fixInitialSelection( root, nativeSel, doFocus ) {
-		// It may happen that setting proper selection will
-		// cause focus to be fired (even without actually focusing root).
-		// Cancel it because focus shouldn't be fired when retriving selection. (#10115)
-		var listener = root.on( 'focus', function( evt ) {
-			evt.cancel();
-		}, null, null, -100 );
+	function moveNativeSelectionToBookmark( document, bm ) {
+		var sel = document.getSelection(),
+			range = document.createRange();
 
-		// FF && Webkit.
-		if ( !CKEDITOR.env.ie ) {
-			var range = new CKEDITOR.dom.range( root );
-			range.moveToElementEditStart( root );
-
-			var nativeRange = root.getDocument().$.createRange();
-			nativeRange.setStart( range.startContainer.$, range.startOffset );
-			nativeRange.collapse( 1 );
-
-			nativeSel.removeAllRanges();
-			nativeSel.addRange( nativeRange );
-		}
-		else {
-			// IE in specific case may also fire selectionchange.
-			// We cannot block bubbling selectionchange, so at least we
-			// can prevent from falling into inf recursion caused by fix for #9699
-			// (see wysiwygarea plugin).
-			// http://dev.ckeditor.com/ticket/10438#comment:13
-			var listener2 = root.getDocument().on( 'selectionchange', function( evt ) {
-				evt.cancel();
-			}, null, null, -100 );
-		}
-
-		doFocus && root.focus();
-
-		listener.removeListener();
-		listener2 && listener2.removeListener();
+		range.setStart( bm[ 0 ].node, bm[ 0 ].offset );
+		range.collapse( true );
+		sel.removeAllRanges();
+		sel.addRange( range );
+		sel.extend( bm[ 1 ].node, bm[ 1 ].offset );
 	}
 
 	// Creates cke_hidden_sel container and puts real selection there.
@@ -560,16 +542,16 @@
 				restoreSel = 0;
 			} );
 
+			// Save a cloned version of current selection.
+			function saveSel() {
+				lastSel = new CKEDITOR.dom.selection( editor.getSelection() );
+				lastSel.lock();
+			}
+
 			// Browsers could loose the selection once the editable lost focus,
 			// in such case we need to reproduce it by saving a locked selection
 			// and restoring it upon focus gain.
 			if ( CKEDITOR.env.ie || isInline ) {
-				// Save a cloned version of current selection.
-				function saveSel() {
-					lastSel = new CKEDITOR.dom.selection( editor.getSelection() );
-					lastSel.lock();
-				}
-
 				// For old IEs, we can retrieve the last correct DOM selection upon the "beforedeactivate" event.
 				// For the rest, a more frequent check is required for each selection change made.
 				if ( isMSSelection )
@@ -619,11 +601,6 @@
 				// the normal behavior on old IEs. (#1659, #7932)
 				if ( doc.$.compatMode != 'BackCompat' ) {
 					if ( CKEDITOR.env.ie7Compat || CKEDITOR.env.ie6Compat ) {
-						function moveRangeToPoint( range, x, y ) {
-							// Error prune in IE7. (#9034, #9110)
-							try { range.moveToPoint( x, y ); } catch ( e ) {}
-						}
-
 						html.on( 'mousedown', function( evt ) {
 							evt = evt.data;
 
@@ -652,7 +629,6 @@
 							}
 
 							function onSelectEnd() {
-
 								html.removeListener( 'mousemove', onHover );
 								removeListeners();
 
@@ -664,8 +640,8 @@
 							// We're sure that the click happens at the region
 							// below body, but not on scrollbar.
 							if ( evt.getTarget().is( 'html' ) &&
-									 evt.$.y < html.$.clientHeight &&
-									 evt.$.x < html.$.clientWidth ) {
+									evt.$.y < html.$.clientHeight &&
+									evt.$.x < html.$.clientWidth ) {
 								// Start to build the text range.
 								var textRng = body.$.createTextRange();
 								moveRangeToPoint( textRng, evt.$.clientX, evt.$.clientY );
@@ -689,26 +665,7 @@
 								outerDoc.on( 'mouseup', onSelectEnd );
 								html.on( 'mouseup', onSelectEnd );
 							}
-
 						} );
-
-						function removeListeners() {
-							outerDoc.removeListener( 'mouseup', onSelectEnd );
-							html.removeListener( 'mouseup', onSelectEnd );
-						}
-
-						function onSelectEnd() {
-							removeListeners();
-
-							// The event is not fired when clicking on the scrollbars,
-							// so we can safely check the following to understand
-							// whether the empty space following <body> has been clicked.
-								var sel = CKEDITOR.document.$.selection,
-									range = sel.createRange();
-								// The selection range is reported on host, but actually it should applies to the content doc.
-								if ( sel.type != 'None' && range.parentElement().ownerDocument == doc.$ )
-									range.select();
-						}
 					}
 				}
 			}
@@ -746,8 +703,9 @@
 			// Use document instead of editable in non-IEs for observing mouseup
 			// since editable won't fire the event if selection process started within iframe and ended out
 			// of the editor (#9851).
-			else
+			else {
 				editable.attachListener( CKEDITOR.env.ie ? editable : doc.getDocumentElement(), 'mouseup', checkSelectionChangeTimeout, editor );
+			}
 
 			if ( CKEDITOR.env.webkit ) {
 				// Before keystroke is handled by editor, check to remove the filling char.
@@ -775,6 +733,32 @@
 			// Automatically select non-editable element when navigating into
 			// it by left/right or backspace/del keys.
 			editable.attachListener( editable, 'keydown', getOnKeyDownListener( editor ), null, null, -1 );
+
+			function moveRangeToPoint( range, x, y ) {
+				// Error prune in IE7. (#9034, #9110)
+				try {
+					range.moveToPoint( x, y );
+				} catch ( e ) {}
+			}
+
+			function removeListeners() {
+				outerDoc.removeListener( 'mouseup', onSelectEnd );
+				html.removeListener( 'mouseup', onSelectEnd );
+			}
+
+			function onSelectEnd() {
+				removeListeners();
+
+				// The event is not fired when clicking on the scrollbars,
+				// so we can safely check the following to understand
+				// whether the empty space following <body> has been clicked.
+				var sel = CKEDITOR.document.$.selection,
+					range = sel.createRange();
+
+				// The selection range is reported on host, but actually it should applies to the content doc.
+				if ( sel.type != 'None' && range.parentElement().ownerDocument == doc.$ )
+					range.select();
+			}
 		} );
 
 		editor.on( 'setData', function() {
@@ -808,14 +792,32 @@
 
 		// When loaded data are ready check whether hidden selection container was not loaded.
 		editor.on( 'loadSnapshot', function() {
-			// TODO replace with el.find() which will be introduced in #9764,
-			// because it may happen that hidden sel container won't be the last element.
-			var el = editor.editable().getLast( function( node ) {
-				return node.type == CKEDITOR.NODE_ELEMENT;
-			} );
+			var isElement = CKEDITOR.dom.walker.nodeType( CKEDITOR.NODE_ELEMENT ),
+				// TODO replace with el.find() which will be introduced in #9764,
+				// because it may happen that hidden sel container won't be the last element.
+				last = editor.editable().getLast( isElement );
 
-			if ( el && el.hasAttribute( 'data-cke-hidden-sel' ) )
-				el.remove();
+			if ( last && last.hasAttribute( 'data-cke-hidden-sel' ) ) {
+				last.remove();
+
+				// Firefox does a very unfortunate thing. When a non-editable element is the only
+				// element in the editable, when we remove the hidden selection container, Firefox
+				// will insert a bogus <br> at the beginning of the editable...
+				// See: https://bugzilla.mozilla.org/show_bug.cgi?id=911201
+				//
+				// This behavior is never desired because this <br> pushes the content lower, but in
+				// this case it is especially dangerous, because it happens when a bookmark is being restored.
+				// Since this <br> is inserted at the beginning it changes indexes and thus breaks the bookmark2
+				// what results in errors.
+				//
+				// So... let's revert what Firefox broke.
+				if ( CKEDITOR.env.gecko ) {
+					var first = editor.editable().getFirst( isElement );
+					if ( first && first.is( 'br' ) && first.getAttribute( '_moz_editor_bogus_node' ) ) {
+						first.remove();
+					}
+				}
+			}
 		}, null, null, 100 );
 
 		editor.on( 'key', function( evt ) {
@@ -838,7 +840,9 @@
 	} );
 
 	CKEDITOR.on( 'instanceReady', function( evt ) {
-		var editor = evt.editor;
+		var editor = evt.editor,
+			fillingCharBefore,
+			selectionBookmark;
 
 		// On WebKit only, we need a special "filling" char on some situations
 		// (#1272). Here we set the events that should invalidate that char.
@@ -850,49 +854,49 @@
 				removeFillingChar( editor.editable() );
 			}, null, null, -1 );
 
-			var fillingCharBefore, resetSelection;
-
-			function beforeData() {
-				var editable = editor.editable();
-				if ( !editable )
-					return;
-
-				var fillingChar = getFillingChar( editable );
-
-				if ( fillingChar ) {
-					// If cursor is right blinking by side of the filler node, save it for restoring,
-					// as the following text substitution will blind it. (#7437)
-					var sel = editor.document.$.defaultView.getSelection();
-					if ( sel.type == 'Caret' && sel.anchorNode == fillingChar.$ )
-						resetSelection = 1;
-
-					fillingCharBefore = fillingChar.getText();
-					fillingChar.setText( replaceFillingChar( fillingCharBefore ) );
-				}
-			}
-
-			function afterData() {
-				var editable = editor.editable();
-				if ( !editable )
-					return;
-
-				var fillingChar = getFillingChar( editable );
-
-				if ( fillingChar ) {
-					fillingChar.setText( fillingCharBefore );
-
-					if ( resetSelection ) {
-						editor.document.$.defaultView.getSelection().setPosition( fillingChar.$, fillingChar.getLength() );
-						resetSelection = 0;
-					}
-				}
-			}
-
 			editor.on( 'beforeUndoImage', beforeData );
 			editor.on( 'afterUndoImage', afterData );
 			editor.on( 'beforeGetData', beforeData, null, null, 0 );
 			editor.on( 'getData', afterData );
 		}
+
+		function beforeData() {
+			var editable = editor.editable();
+			if ( !editable )
+				return;
+
+			var fillingChar = getFillingChar( editable );
+
+			if ( fillingChar ) {
+				// If the selection's focus or anchor is located in the filling char's text node,
+				// we need to restore the selection in afterData, because it will be lost
+				// when setting text. Selection's direction must be preserved.
+				// (#7437, #12489, #12491 comment:3)
+				var sel = editor.document.$.getSelection();
+				if ( sel.type != 'None' && ( sel.anchorNode == fillingChar.$ || sel.focusNode == fillingChar.$ ) )
+					selectionBookmark = createNativeSelectionBookmark( sel );
+
+				fillingCharBefore = fillingChar.getText();
+				fillingChar.setText( replaceFillingChar( fillingCharBefore ) );
+			}
+		}
+
+		function afterData() {
+			var editable = editor.editable();
+			if ( !editable )
+				return;
+
+			var fillingChar = getFillingChar( editable );
+
+			if ( fillingChar ) {
+				fillingChar.setText( fillingCharBefore );
+
+				if ( selectionBookmark ) {
+					moveNativeSelectionToBookmark( editor.document.$, selectionBookmark );
+					selectionBookmark = null;
+				}
+			}
+		}
 	} );
 
 	/**
@@ -1110,47 +1114,6 @@
 			return this;
 		}
 
-		// On WebKit, it may happen that we've already have focus
-		// on the editable element while still having no selection
-		// available. We normalize it here by replicating the
-		// behavior of other browsers.
-		//
-		// Webkit's condition covers also the case when editable hasn't been focused
-		// at all. Thanks to this hack Webkit always has selection in the right place.
-		//
-		// On FF and IE we only fix the first case, when editable was activated
-		// but the selection is broken - usually this happens after setData if editor was focused.
-
-		var sel = isMSSelection ? this.document.$.selection : this.document.getWindow().$.getSelection();
-
-		if ( CKEDITOR.env.webkit ) {
-			if ( sel.type == 'None' && this.document.getActive().equals( root ) || sel.type == 'Caret' && sel.anchorNode.nodeType == CKEDITOR.NODE_DOCUMENT )
-				fixInitialSelection( root, sel );
-		}
-		else if ( CKEDITOR.env.gecko ) {
-			if ( sel && this.document.getActive().equals( root ) &&
-				sel.anchorNode && sel.anchorNode.nodeType == CKEDITOR.NODE_DOCUMENT )
-				fixInitialSelection( root, sel, true );
-		}
-		else if ( CKEDITOR.env.ie ) {
-			var active = this.document.getActive();
-
-			// IEs 9+.
-			if ( !isMSSelection ) {
-				var anchorNode = sel && sel.anchorNode;
-
-				if ( anchorNode )
-					anchorNode = new CKEDITOR.dom.node( anchorNode );
-
-				if ( active && active.equals( this.document.getDocumentElement() ) &&
-					anchorNode && ( root.equals( anchorNode ) || root.contains( anchorNode ) ) )
-					fixInitialSelection( root, null, true );
-			}
-			// IEs 7&8.
-			else if ( sel.type == 'None' && active && active.equals( this.document.getDocumentElement() ) )
-				fixInitialSelection( root, null, true );
-		}
-
 		// Check whether browser focus is really inside of the editable element.
 
 		var nativeSel = this.getNative(),
@@ -1273,8 +1236,11 @@
 				var range = sel.getRangeAt( 0 ),
 					startContainer = range.startContainer;
 
-				if ( startContainer == range.endContainer && startContainer.nodeType == 1 && ( range.endOffset - range.startOffset ) == 1 && styleObjectElements[ startContainer.childNodes[ range.startOffset ].nodeName.toLowerCase() ] )
+				if ( startContainer == range.endContainer && startContainer.nodeType == 1 &&
+					( range.endOffset - range.startOffset ) == 1 &&
+					styleObjectElements[ startContainer.childNodes[ range.startOffset ].nodeName.toLowerCase() ] ) {
 					type = CKEDITOR.SELECTION_ELEMENT;
+				}
 
 			}
 
@@ -1303,113 +1269,112 @@
 				// Finds the container and offset for a specific boundary
 				// of an IE range.
 				var getBoundaryInformation = function( range, start ) {
-						// Creates a collapsed range at the requested boundary.
-						range = range.duplicate();
-						range.collapse( start );
+					// Creates a collapsed range at the requested boundary.
+					range = range.duplicate();
+					range.collapse( start );
 
-						// Gets the element that encloses the range entirely.
-						var parent = range.parentElement(),
-							doc = parent.ownerDocument;
+					// Gets the element that encloses the range entirely.
+					var parent = range.parentElement();
 
-						// Empty parent element, e.g. <i>^</i>
-						if ( !parent.hasChildNodes() )
-							return { container: parent, offset: 0 };
+					// Empty parent element, e.g. <i>^</i>
+					if ( !parent.hasChildNodes() )
+						return { container: parent, offset: 0 };
 
-						var siblings = parent.children,
-							child, sibling,
-							testRange = range.duplicate(),
-							startIndex = 0,
-							endIndex = siblings.length - 1,
-							index = -1,
-							position, distance, container;
+					var siblings = parent.children,
+						child, sibling,
+						testRange = range.duplicate(),
+						startIndex = 0,
+						endIndex = siblings.length - 1,
+						index = -1,
+						position, distance, container;
 
-						// Binary search over all element childs to test the range to see whether
-						// range is right on the boundary of one element.
-						while ( startIndex <= endIndex ) {
-							index = Math.floor( ( startIndex + endIndex ) / 2 );
-							child = siblings[ index ];
-							testRange.moveToElementText( child );
-							position = testRange.compareEndPoints( 'StartToStart', range );
+					// Binary search over all element childs to test the range to see whether
+					// range is right on the boundary of one element.
+					while ( startIndex <= endIndex ) {
+						index = Math.floor( ( startIndex + endIndex ) / 2 );
+						child = siblings[ index ];
+						testRange.moveToElementText( child );
+						position = testRange.compareEndPoints( 'StartToStart', range );
 
-							if ( position > 0 )
-								endIndex = index - 1;
-							else if ( position < 0 )
-								startIndex = index + 1;
+						if ( position > 0 )
+							endIndex = index - 1;
+						else if ( position < 0 )
+							startIndex = index + 1;
+						else
+							return { container: parent, offset: getNodeIndex( child ) };
+					}
+
+					// All childs are text nodes,
+					// or to the right hand of test range are all text nodes. (#6992)
+					if ( index == -1 || index == siblings.length - 1 && position < 0 ) {
+						// Adapt test range to embrace the entire parent contents.
+						testRange.moveToElementText( parent );
+						testRange.setEndPoint( 'StartToStart', range );
+
+						// IE report line break as CRLF with range.text but
+						// only LF with textnode.nodeValue, normalize them to avoid
+						// breaking character counting logic below. (#3949)
+						distance = testRange.text.replace( /(\r\n|\r)/g, '\n' ).length;
+
+						siblings = parent.childNodes;
+
+						// Actual range anchor right beside test range at the boundary of text node.
+						if ( !distance ) {
+							child = siblings[ siblings.length - 1 ];
+
+							if ( child.nodeType != CKEDITOR.NODE_TEXT )
+								return { container: parent, offset: siblings.length };
 							else
-								return { container: parent, offset: getNodeIndex( child ) };
+								return { container: child, offset: child.nodeValue.length };
 						}
 
-						// All childs are text nodes,
-						// or to the right hand of test range are all text nodes. (#6992)
-						if ( index == -1 || index == siblings.length - 1 && position < 0 ) {
-							// Adapt test range to embrace the entire parent contents.
-							testRange.moveToElementText( parent );
-							testRange.setEndPoint( 'StartToStart', range );
-
-							// IE report line break as CRLF with range.text but
-							// only LF with textnode.nodeValue, normalize them to avoid
-							// breaking character counting logic below. (#3949)
-							distance = testRange.text.replace( /(\r\n|\r)/g, '\n' ).length;
-
-							siblings = parent.childNodes;
-
-							// Actual range anchor right beside test range at the boundary of text node.
-							if ( !distance ) {
-								child = siblings[ siblings.length - 1 ];
-
-								if ( child.nodeType != CKEDITOR.NODE_TEXT )
-									return { container: parent, offset: siblings.length };
-								else
-									return { container: child, offset: child.nodeValue.length };
+						// Start the measuring until distance overflows, meanwhile count the text nodes.
+						var i = siblings.length;
+						while ( distance > 0 && i > 0 ) {
+							sibling = siblings[ --i ];
+							if ( sibling.nodeType == CKEDITOR.NODE_TEXT ) {
+								container = sibling;
+								distance -= sibling.nodeValue.length;
 							}
+						}
 
-							// Start the measuring until distance overflows, meanwhile count the text nodes.
-							var i = siblings.length;
-							while ( distance > 0 && i > 0 ) {
-								sibling = siblings[ --i ];
+						return { container: container, offset: -distance };
+					}
+					// Test range was one offset beyond OR behind the anchored text node.
+					else {
+						// Adapt one side of test range to the actual range
+						// for measuring the offset between them.
+						testRange.collapse( position > 0 ? true : false );
+						testRange.setEndPoint( position > 0 ? 'StartToStart' : 'EndToStart', range );
+
+						// IE report line break as CRLF with range.text but
+						// only LF with textnode.nodeValue, normalize them to avoid
+						// breaking character counting logic below. (#3949)
+						distance = testRange.text.replace( /(\r\n|\r)/g, '\n' ).length;
+
+						// Actual range anchor right beside test range at the inner boundary of text node.
+						if ( !distance )
+							return { container: parent, offset: getNodeIndex( child ) + ( position > 0 ? 0 : 1 ) };
+
+						// Start the measuring until distance overflows, meanwhile count the text nodes.
+						while ( distance > 0 ) {
+							try {
+								sibling = child[ position > 0 ? 'previousSibling' : 'nextSibling' ];
 								if ( sibling.nodeType == CKEDITOR.NODE_TEXT ) {
-									container = sibling;
 									distance -= sibling.nodeValue.length;
+									container = sibling;
 								}
+								child = sibling;
 							}
-
-							return { container: container, offset: -distance };
-						}
-						// Test range was one offset beyond OR behind the anchored text node.
-						else {
-							// Adapt one side of test range to the actual range
-							// for measuring the offset between them.
-							testRange.collapse( position > 0 ? true : false );
-							testRange.setEndPoint( position > 0 ? 'StartToStart' : 'EndToStart', range );
-
-							// IE report line break as CRLF with range.text but
-							// only LF with textnode.nodeValue, normalize them to avoid
-							// breaking character counting logic below. (#3949)
-							distance = testRange.text.replace( /(\r\n|\r)/g, '\n' ).length;
-
-							// Actual range anchor right beside test range at the inner boundary of text node.
-							if ( !distance )
-								return { container: parent, offset: getNodeIndex( child ) + ( position > 0 ? 0 : 1 ) };
-
-							// Start the measuring until distance overflows, meanwhile count the text nodes.
-							while ( distance > 0 ) {
-								try {
-									sibling = child[ position > 0 ? 'previousSibling' : 'nextSibling' ];
-									if ( sibling.nodeType == CKEDITOR.NODE_TEXT ) {
-										distance -= sibling.nodeValue.length;
-										container = sibling;
-									}
-									child = sibling;
-								}
-								// Measurement in IE could be somtimes wrong because of <select> element. (#4611)
-								catch ( e ) {
-									return { container: parent, offset: getNodeIndex( child ) };
-								}
+							// Measurement in IE could be somtimes wrong because of <select> element. (#4611)
+							catch ( e ) {
+								return { container: parent, offset: getNodeIndex( child ) };
 							}
-
-							return { container: container, offset: position > 0 ? -distance : container.nodeValue.length + distance };
 						}
-					};
+
+						return { container: container, offset: position > 0 ? -distance : container.nodeValue.length + distance };
+					}
+				};
 
 				return function() {
 					// IE doesn't have range support (in the W3C way), so we
@@ -1449,7 +1414,7 @@
 							range = new CKEDITOR.dom.range( this.root );
 
 							for ( ; j < parentElement.childNodes.length && parentElement.childNodes[ j ] != element; j++ ) {
-								/*jsl:pass*/
+
 							}
 
 							range.setStart( new CKEDITOR.dom.node( parentElement ), j );
@@ -1462,30 +1427,30 @@
 
 					return [];
 				};
-			} )() : function() {
+			} )() :
+			function() {
+				// On browsers implementing the W3C range, we simply
+				// tranform the native ranges in CKEDITOR.dom.range
+				// instances.
 
-					// On browsers implementing the W3C range, we simply
-					// tranform the native ranges in CKEDITOR.dom.range
-					// instances.
+				var ranges = [],
+					range,
+					sel = this.getNative();
 
-					var ranges = [],
-						range,
-						sel = this.getNative();
-
-					if ( !sel )
-						return ranges;
-
-					for ( var i = 0; i < sel.rangeCount; i++ ) {
-						var nativeRange = sel.getRangeAt( i );
-
-						range = new CKEDITOR.dom.range( this.root );
-
-						range.setStart( new CKEDITOR.dom.node( nativeRange.startContainer ), nativeRange.startOffset );
-						range.setEnd( new CKEDITOR.dom.node( nativeRange.endContainer ), nativeRange.endOffset );
-						ranges.push( range );
-					}
+				if ( !sel )
 					return ranges;
-				};
+
+				for ( var i = 0; i < sel.rangeCount; i++ ) {
+					var nativeRange = sel.getRangeAt( i );
+
+					range = new CKEDITOR.dom.range( this.root );
+
+					range.setStart( new CKEDITOR.dom.node( nativeRange.startContainer ), nativeRange.startOffset );
+					range.setEnd( new CKEDITOR.dom.node( nativeRange.endContainer ), nativeRange.endOffset );
+					ranges.push( range );
+				}
+				return ranges;
+			};
 
 			return function( onlyEditables ) {
 				var cache = this._.cache,
@@ -1693,8 +1658,7 @@
 			this._.cache = {};
 			this.isFake = 0;
 
-			var editor = this.root.editor,
-				listener;
+			var editor = this.root.editor;
 
 			// Invalidate any fake selection available in the editor.
 			if ( editor && editor._.fakeSelection ) {
@@ -1705,11 +1669,9 @@
 
 					removeHiddenSelectionContainer( editor );
 				}
-				// TODO after #9786 use commented out lines instead of console.log.
-				else // %REMOVE_LINE%
-					window.console && console.log( 'Wrong selection instance resets fake selection.' ); // %REMOVE_LINE%
-				// else // %REMOVE_LINE%
-				//	CKEDITOR.debug.error( 'Wrong selection instance resets fake selection.', CKEDITOR.DEBUG_CRITICAL ); // %REMOVE_LINE%
+				else {
+					CKEDITOR.warn( 'selection-fake-reset' );
+				}
 			}
 
 			this.rev = nextRev++;
@@ -1796,7 +1758,8 @@
 				// Try to make a object selection, be careful with selecting phase element in IE
 				// will breaks the selection in non-framed environment.
 				var selected = range.getEnclosedNode();
-				if ( selected && selected.type == CKEDITOR.NODE_ELEMENT && selected.getName() in styleObjectElements && !( selected.is( 'a' ) && selected.getText() ) ) {
+				if ( selected && selected.type == CKEDITOR.NODE_ELEMENT && selected.getName() in styleObjectElements &&
+					!( selected.is( 'a' ) && selected.getText() ) ) {
 					try {
 						ieRange = selected.$.createControlRange();
 						ieRange.addElement( selected.$ );
@@ -1807,7 +1770,8 @@
 
 				// IE doesn't support selecting the entire table row/cell, move the selection into cells, e.g.
 				// <table><tbody><tr>[<td>cell</b></td>... => <table><tbody><tr><td>[cell</td>...
-				if ( range.startContainer.type == CKEDITOR.NODE_ELEMENT && range.startContainer.getName() in nonCells || range.endContainer.type == CKEDITOR.NODE_ELEMENT && range.endContainer.getName() in nonCells ) {
+				if ( range.startContainer.type == CKEDITOR.NODE_ELEMENT && range.startContainer.getName() in nonCells ||
+					range.endContainer.type == CKEDITOR.NODE_ELEMENT && range.endContainer.getName() in nonCells ) {
 					range.shrink( CKEDITOR.NODE_ELEMENT, true );
 					// The range might get collapsed (#7975). Update cached variable.
 					collapsed = range.collapsed;
@@ -1846,8 +1810,8 @@
 					// situations, but just on those needed.
 					var next = startNode.getNext( notWhitespaces );
 					var inPre = startNode.hasAscendant( 'pre' );
-					isStartMarkerAlone = ( !( next && next.getText && next.getText().match( fillerTextRegex ) ) // already a filler there?
-					&& ( inPre || !startNode.hasPrevious() || ( startNode.getPrevious().is && startNode.getPrevious().is( 'br' ) ) ) );
+					isStartMarkerAlone = ( !( next && next.getText && next.getText().match( fillerTextRegex ) ) && // already a filler there?
+						( inPre || !startNode.hasPrevious() || ( startNode.getPrevious().is && startNode.getPrevious().is( 'br' ) ) ) );
 
 					// Append a temporary <span>&#65279;</span> before the selection.
 					// This is needed to avoid IE destroying selections inside empty
@@ -1880,8 +1844,9 @@
 
 						// Remove our temporary stuff.
 						range.document.$.selection.clear();
-					} else
+					} else {
 						ieRange.select();
+					}
 
 					range.moveToPosition( dummySpan, CKEDITOR.POSITION_BEFORE_START );
 					dummySpan.remove();
@@ -1930,7 +1895,6 @@
 					range = ranges[ i ];
 
 					var nativeRange = this.document.$.createRange();
-					var startContainer = range.startContainer;
 
 					if ( range.collapsed && CKEDITOR.env.webkit && rangeRequiresFix( range ) ) {
 						// Append a zero-width space so WebKit will not try to
@@ -1946,8 +1910,9 @@
 						if ( next && !fillingChar.getPrevious() && next.type == CKEDITOR.NODE_ELEMENT && next.getName() == 'br' ) {
 							removeFillingChar( this.root );
 							range.moveToPosition( next, CKEDITOR.POSITION_BEFORE_START );
-						} else
+						} else {
 							range.moveToPosition( fillingChar, CKEDITOR.POSITION_AFTER_END );
+						}
 					}
 
 					nativeRange.setStart( range.startContainer.$, range.startOffset );
@@ -1961,8 +1926,9 @@
 						if ( e.toString().indexOf( 'NS_ERROR_ILLEGAL_VALUE' ) >= 0 ) {
 							range.collapse( 1 );
 							nativeRange.setEnd( range.endContainer.$, range.endOffset );
-						} else
+						} else {
 							throw e;
+						}
 					}
 
 					// Select the range.
@@ -2085,15 +2051,26 @@
 		 * @returns {CKEDITOR.dom.selection} This selection object, after the ranges were selected.
 		 */
 		selectBookmarks: function( bookmarks ) {
-			var ranges = [];
+			var ranges = [],
+				node;
+
 			for ( var i = 0; i < bookmarks.length; i++ ) {
 				var range = new CKEDITOR.dom.range( this.root );
 				range.moveToBookmark( bookmarks[ i ] );
 				ranges.push( range );
 			}
 
+			// It may happen that the content change during loading, before selection is set so bookmark leads to text node.
+			if ( bookmarks.isFake ) {
+				node = ranges[ 0 ].getEnclosedNode();
+				if ( !node || node.type != CKEDITOR.NODE_ELEMENT ) {
+					CKEDITOR.warn( 'selection-not-fake' );
+					bookmarks.isFake = 0;
+				}
+			}
+
 			if ( bookmarks.isFake )
-				this.fake( ranges[ 0 ].getEnclosedNode() );
+				this.fake( node );
 			else
 				this.selectRanges( ranges );
 
@@ -2123,7 +2100,6 @@
 		 *		editor.getSelection().scrollIntoView();
 		 */
 		scrollIntoView: function() {
-
 			// Scrolls the first range into view.
 			if ( this.type != CKEDITOR.SELECTION_NONE )
 				this.getRanges()[ 0 ].scrollIntoView();
@@ -2139,8 +2115,9 @@
 
 			var nativ = this.getNative();
 
-			try { nativ && nativ[ isMSSelection ? 'empty' : 'removeAllRanges' ](); }
-			catch ( er ) {}
+			try {
+				nativ && nativ[ isMSSelection ? 'empty' : 'removeAllRanges' ]();
+			} catch ( er ) {}
 
 			this.reset();
 		}
