@@ -1,12 +1,13 @@
 /**
- * @license Copyright (c) 2003-2014, CKSource - Frederico Knabben. All rights reserved.
+ * @license Copyright (c) 2003-2015, CKSource - Frederico Knabben. All rights reserved.
  * For licensing, see LICENSE.md or http://ckeditor.com/license
  */
 
 CKEDITOR.plugins.setLang( 'codesnippet', 'de', {
 	button: 'Codeschnipsel einfügen',
 	codeContents: 'Codeinhalt',
-	emptySnippetError: 'Ein Codeschnipsel kann nicht leer sein.',
+	emptySnippetError: 'Ein Codeschnipsel darf nicht leer sein.',
 	language: 'Sprache',
-	title: 'Codeschnipsel'
+	title: 'Codeschnipsel',
+	pathName: 'Codeschnipsel'
 } );
