@@ -1,10 +1,10 @@
-﻿/**
- * @license Copyright (c) 2003-2014, CKSource - Frederico Knabben. All rights reserved.
+/**
+ * @license Copyright (c) 2003-2015, CKSource - Frederico Knabben. All rights reserved.
  * For licensing, see LICENSE.md or http://ckeditor.com/license
  */
 
 CKEDITOR.plugins.add( 'devtools', {
-	lang: 'ar,bg,ca,cs,cy,da,de,el,en,en-gb,eo,es,et,eu,fa,fi,fr,fr-ca,gl,gu,he,hr,hu,id,it,ja,km,ko,ku,lt,lv,nb,nl,no,pl,pt,pt-br,ru,si,sk,sl,sq,sv,tr,tt,ug,uk,vi,zh,zh-cn', // %REMOVE_LINE_CORE%
+	lang: 'ar,bg,ca,cs,cy,da,de,el,en,en-gb,eo,es,et,eu,fa,fi,fr,fr-ca,gl,gu,he,hr,hu,id,it,ja,km,ko,ku,lt,lv,nb,nl,no,pl,pt,pt-br,ro,ru,si,sk,sl,sq,sv,tr,tt,ug,uk,vi,zh,zh-cn', // %REMOVE_LINE_CORE%
 
 	init: function( editor ) {
 		editor._.showDialogDefinitionTooltips = 1;
@@ -46,8 +46,9 @@ CKEDITOR.plugins.add( 'devtools', {
 		if ( editor.lang.dir == 'rtl' ) {
 			var viewPaneSize = CKEDITOR.document.getWindow().getViewPaneSize();
 			styles.right = ( viewPaneSize.width - pos.x - el.getSize( 'width' ) ) + 'px';
-		} else
+		} else {
 			styles.left = pos.x + 'px';
+		}
 
 		tooltip.setStyles( styles );
 	}
@@ -113,6 +114,9 @@ CKEDITOR.plugins.add( 'devtools', {
  * A function that returns the text to be displayed inside the Developer Tools
  * tooltip when hovering over a dialog UI element.
  *
+ * Read more in the [documentation](#!/guide/dev_devtools)
+ * and see the [SDK sample](http://sdk.ckeditor.com/samples/devtools.html).
+ *
  *		// This is actually the default value.
  *		// Show dialog window name, tab ID, and element ID.
  *		config.devtools_textCallback = function( editor, dialog, element, tabName ) {
@@ -146,6 +150,9 @@ CKEDITOR.plugins.add( 'devtools', {
 /**
  * A setting that stores CSS rules to be injected into the page with styles to be applied to the tooltip element.
  *
+ * Read more in the [documentation](#!/guide/dev_devtools)
+ * and see the [SDK sample](http://sdk.ckeditor.com/samples/devtools.html).
+ *
  *		// This is actually the default value.
  *		CKEDITOR.config.devtools_styles =
  *			'#cke_tooltip { padding: 5px; border: 2px solid #333; background: #ffffff }' +
