@@ -1,8 +1,8 @@
 ﻿/*
-Copyright (c) 2003-2014, CKSource - Frederico Knabben. All rights reserved.
+Copyright (c) 2003-2015, CKSource - Frederico Knabben. All rights reserved.
 For licensing, see LICENSE.md or http://ckeditor.com/license
 */
 CKEDITOR.plugins.setLang( 'elementspath', 'eu', {
-	eleLabel: 'Elementu bidea',
+	eleLabel: 'Elementuen bidea',
 	eleTitle: '%1 elementua'
 } );
