@@ -1,11 +1,11 @@
 ﻿/*
-Copyright (c) 2003-2014, CKSource - Frederico Knabben. All rights reserved.
+Copyright (c) 2003-2015, CKSource - Frederico Knabben. All rights reserved.
 For licensing, see LICENSE.md or http://ckeditor.com/license
 */
 CKEDITOR.plugins.setLang( 'image2', 'ug', {
 	alt: 'تېكىست ئالماشتۇر',
 	btnUpload: 'مۇلازىمېتىرغا يۈكلە',
-	captioned: 'Captioned image', // MISSING
+	captioned: 'ماۋزۇلۇق سۈرەت',
 	captionPlaceholder: 'Caption', // MISSING
 	infoTab: 'سۈرەت',
 	lockRatio: 'نىسبەتنى قۇلۇپلا',
