@@ -1,20 +1,17 @@
 ﻿/**
- * @license Copyright (c) 2003-2014, CKSource - Frederico Knabben. All rights reserved.
+ * @license Copyright (c) 2003-2015, CKSource - Frederico Knabben. All rights reserved.
  * For licensing, see LICENSE.md or http://ckeditor.com/license
  */
 
 /**
- * @fileOverview [Mathematical Formulas](http://ckeditor.com/addon/mathjax) plugin.
+ * @fileOverview The [Mathematical Formulas](http://ckeditor.com/addon/mathjax) plugin that allows you to create and modify mathematical equations written in TeX directly in CKEditor..
  */
 
 'use strict';
 
 ( function() {
-
-	var cdn = 'http:\/\/cdn.mathjax.org\/mathjax\/2.2-latest\/MathJax.js?config=TeX-AMS_HTML';
-
 	CKEDITOR.plugins.add( 'mathjax', {
-		lang: 'ar,ca,cs,cy,de,el,en,en-gb,eo,es,fa,fi,fr,gl,he,hr,hu,it,ja,km,nb,nl,no,pl,pt,pt-br,ro,ru,sl,sv,tt,uk,vi,zh,zh-cn', // %REMOVE_LINE_CORE%
+		lang: 'af,ar,bg,ca,cs,cy,da,de,el,en,en-gb,eo,es,eu,fa,fi,fr,gl,he,hr,hu,id,it,ja,km,ko,ku,lt,nb,nl,no,pl,pt,pt-br,ro,ru,sk,sl,sq,sv,tr,tt,ug,uk,vi,zh,zh-cn', // %REMOVE_LINE_CORE%
 		requires: 'widget,dialog',
 		icons: 'mathjax',
 		hidpi: true, // %REMOVE_LINE_CORE%
@@ -22,6 +19,10 @@
 		init: function( editor ) {
 			var cls = editor.config.mathJaxClass || 'math-tex';
 
+			if ( !editor.config.mathJaxLib ) {
+				CKEDITOR.error( 'mathjax-no-config' );
+			}
+
 			editor.widgets.add( 'mathjax', {
 				inline: true,
 				dialog: 'mathjax',
@@ -127,8 +128,10 @@
 
 			// Add MathJax script to page preview.
 			editor.on( 'contentPreview', function( evt ) {
-				evt.data.dataValue = evt.data.dataValue.replace( /<\/head>/,
-					'<script src="' + ( editor.config.mathJaxLib ? CKEDITOR.getUrl( editor.config.mathJaxLib ) : cdn ) + '"><\/script><\/head>' );
+				evt.data.dataValue = evt.data.dataValue.replace(
+					/<\/head>/,
+					'<script src="' + CKEDITOR.getUrl( editor.config.mathJaxLib ) + '"><\/script><\/head>'
+				);
 			} );
 
 			editor.on( 'paste', function( evt ) {
@@ -136,7 +139,7 @@
 				// Also iFrame in paste content is reason of "Unspecified error" in IE9 (#10857).
 				var regex = new RegExp( '<span[^>]*?' + cls + '.*?<\/span>', 'ig' );
 				evt.data.dataValue = evt.data.dataValue.replace( regex, function( match ) {
-					return  match.replace( /(<iframe.*?\/iframe>)/i, '' );
+					return match.replace( /(<iframe.*?\/iframe>)/i, '' );
 				} );
 			} );
 		}
@@ -158,16 +161,16 @@
 	 */
 	CKEDITOR.plugins.mathjax.fixSrc =
 		// In Firefox src must exist and be different than about:blank to emit load event.
-		CKEDITOR.env.gecko ? 'javascript:true' :
+		CKEDITOR.env.gecko ? 'javascript:true' : // jshint ignore:line
 		// Support for custom document.domain in IE.
-		CKEDITOR.env.ie ? 'javascript:' +
+		CKEDITOR.env.ie ? 'javascript:' + // jshint ignore:line
 						'void((function(){' + encodeURIComponent(
 							'document.open();' +
 							'(' + CKEDITOR.tools.fixDomain + ')();' +
 							'document.close();'
 						) + '})())' :
 		// In Chrome src must be undefined to emit load event.
-						'javascript:void(0)';
+						'javascript:void(0)'; // jshint ignore:line
 
 	/**
 	 * Loading indicator image generated by http://preloaders.net.
@@ -337,7 +340,7 @@
 								'</script>' +
 
 								// Load MathJax lib.
-								'<script src="' + ( editor.config.mathJaxLib || cdn ) + '"></script>' +
+								'<script src="' + ( editor.config.mathJaxLib ) + '"></script>' +
 							'</head>' +
 							'<body style="padding:0;margin:0;background:transparent;overflow:hidden">' +
 								'<span id="preview"></span>' +
@@ -431,14 +434,20 @@
 } )();
 
 /**
- * Sets the path to the MathJax library. It can be both a local
- * resource and a location different than the default CDN.
+ * Sets the path to the MathJax library. It can be both a local resource and a location different than the default CDN.
  *
  * Please note that this must be a full or absolute path.
  *
- *		config.mathJaxLib = 'http:\/\/example.com\/libs\/MathJax.js';
+ * Read more in the [documentation](#!/guide/dev_mathjax)
+ * and see the [SDK sample](http://sdk.ckeditor.com/samples/mathjax.html).
  *
- * @cfg {String} [mathJaxLib='http:\/\/cdn.mathjax.org\/mathjax\/2.2-latest\/MathJax.js?config=TeX-AMS_HTML']
+ *		config.mathJaxLib = '//cdn.mathjax.org/mathjax/2.2-latest/MathJax.js?config=TeX-AMS_HTML';
+ *
+ * **Note:** Since CKEditor 4.5 this option does not have a default value, so it must
+ * be set in order to enable the MathJax plugin.
+ *
+ * @since 4.3
+ * @cfg {String} mathJaxLib
  * @member CKEDITOR.config
  */
 
@@ -455,6 +464,9 @@
  *
  *		<span class="my-math">\( \sqrt{4} = 2 \)</span>
  *
+ * Read more in the [documentation](#!/guide/dev_mathjax)
+ * and see the [SDK sample](http://sdk.ckeditor.com/samples/mathjax.html).
+ *
  * @cfg {String} [mathJaxClass='math-tex']
  * @member CKEDITOR.config
- */
\ No newline at end of file
+ */
