@@ -1,7 +1,7 @@
 ﻿/*
-Copyright (c) 2003-2014, CKSource - Frederico Knabben. All rights reserved.
+Copyright (c) 2003-2015, CKSource - Frederico Knabben. All rights reserved.
 For licensing, see LICENSE.md or http://ckeditor.com/license
 */
 CKEDITOR.plugins.setLang( 'newpage', 'eu', {
-	toolbar: 'Orrialde Berria'
+	toolbar: 'Orrialde berria'
 } );
