@@ -1,12 +1,12 @@
 /**
- * @license Copyright (c) 2003-2014, CKSource - Frederico Knabben. All rights reserved.
+ * @license Copyright (c) 2003-2015, CKSource - Frederico Knabben. All rights reserved.
  * For licensing, see LICENSE.md or http://ckeditor.com/license
  */
 
 CKEDITOR.plugins.setLang( 'placeholder', 'ug', {
 	title: 'ئورۇن بەلگە خاسلىقى',
 	toolbar: 'ئورۇن بەلگە قۇر',
-	name: 'Placeholder Name', // MISSING
+	name: 'ئورۇن بەلگە ئىسمى',
 	invalidName: 'The placeholder can not be empty and can not contain any of following characters: [, ], <, >', // MISSING
 	pathName: 'placeholder' // MISSING
 } );
